"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var crc32 = require("crc32");
var FileUtil_1 = require("./tool/FileUtil");
var FileCrc32 = /** @class */ (function () {
    function FileCrc32(baseUrl) {
        var self = this;
        var xmUrl = self.getXMUrl(baseUrl);
        var relUrl = path.join(xmUrl, 'bin-release');
        FileUtil_1.FileUtil.rmdirSync(path.join(relUrl, 'resource'));
        var defaultUrl = path.join(xmUrl, 'resource', 'default.res.json');
        var defData = JSON.parse(fs.readFileSync(defaultUrl, 'utf-8'));
        var resources = defData.resources;
        var res;
        for (var i = 0, len = resources.length; i < len; i++) {
            res = resources[i];
            var resHash = '';
            var itemUrl = path.join(xmUrl, 'resource', res.url);
            if (res.type == 'sheet') {
                var filename = FileUtil_1.FileUtil.filename(itemUrl);
                var sheetJson = JSON.parse(fs.readFileSync(itemUrl, 'utf-8'));
                var pngNameArr = sheetJson.file.split('.');
                var hash = self.hashItem(path.join(path.dirname(itemUrl), sheetJson.file));
                sheetJson.file = pngNameArr[0] + '_' + hash + '.' + pngNameArr[1];
                resHash = self.hashItem(itemUrl, new Buffer(JSON.stringify(sheetJson)));
            }
            else {
                resHash = self.hashItem(itemUrl);
            }
            var resUrlArr = res.url.split('.');
            var ext = resUrlArr.pop();
            res.url = resUrlArr.join('.') + '_' + resHash + '.' + ext;
        }
        fs.writeFileSync(defaultUrl.replace(xmUrl, relUrl), new Buffer(JSON.stringify(defData, null, '\t')));
    }
    FileCrc32.prototype.hashItem = function (itemUrl, contents) {
        var self = this;
        if (!contents) {
            contents = fs.readFileSync(itemUrl);
        }
        var xmUrl = self.getXMUrl(itemUrl);
        var relUrl = path.join(xmUrl, 'bin-release');
        var hash = crc32(contents);
        var filename = FileUtil_1.FileUtil.filename(itemUrl);
        var toUrl = filename.replace(xmUrl, relUrl) + '_' + hash + path.extname(itemUrl);
        FileUtil_1.FileUtil.mkdirSync(path.dirname(toUrl));
        fs.writeFileSync(toUrl, contents);
        return hash;
    };
    FileCrc32.prototype.getXMUrl = function (url) {
        return '';
    };
    return FileCrc32;
}());
function run(baseUrl) {
    new FileCrc32(baseUrl);
}
exports.run = run;
